-- SPDX-FileCopyrightText: 2022 CERN (home.cern)
--
-- SPDX-License-Identifier: CERN-OHL-W-2.0+

---------------------------------------------------------------------------------------
-- Title          : Wishbone slave core for Mock Turtle CPU Control/Status registers block
---------------------------------------------------------------------------------------
-- File           : mt_cpu_csr_wb.vhd
-- Author         : auto-generated by wbgen2 from mt_cpu_csr.wb
-- Created        : Mon Jun 11 18:11:01 2018
-- Standard       : VHDL'87
---------------------------------------------------------------------------------------
-- THIS FILE WAS GENERATED BY wbgen2 FROM SOURCE FILE mt_cpu_csr.wb
-- DO NOT HAND-EDIT UNLESS IT'S ABSOLUTELY NECESSARY!
---------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.mt_cpu_csr_wbgen2_pkg.all;


entity mt_cpu_csr_wb_slave is
  port (
    rst_n_i                                  : in     std_logic;
    clk_sys_i                                : in     std_logic;
    wb_adr_i                                 : in     std_logic_vector(6 downto 0);
    wb_dat_i                                 : in     std_logic_vector(31 downto 0);
    wb_dat_o                                 : out    std_logic_vector(31 downto 0);
    wb_cyc_i                                 : in     std_logic;
    wb_sel_i                                 : in     std_logic_vector(3 downto 0);
    wb_stb_i                                 : in     std_logic;
    wb_we_i                                  : in     std_logic;
    wb_ack_o                                 : out    std_logic;
    wb_err_o                                 : out    std_logic;
    wb_rty_o                                 : out    std_logic;
    wb_stall_o                               : out    std_logic;
    ntf_irq_read                             : out    std_logic;
    uart_msg_data_rd_ack_o                   : out    std_logic;
    regs_i                                   : in     t_mt_cpu_csr_in_registers;
    regs_o                                   : out    t_mt_cpu_csr_out_registers
  );
end mt_cpu_csr_wb_slave;

architecture syn of mt_cpu_csr_wb_slave is

signal mt_cpu_csr_reset_int                     : std_logic_vector(7 downto 0);
signal mt_cpu_csr_smem_op_int                   : std_logic_vector(2 downto 0);
signal mt_cpu_csr_hmq_sel_queue_int             : std_logic_vector(2 downto 0);
signal mt_cpu_csr_hmq_sel_core_int              : std_logic_vector(2 downto 0);
signal mt_cpu_csr_hmqi_inten_lo_int             : std_logic_vector(31 downto 0);
signal mt_cpu_csr_hmqi_inten_hi_int             : std_logic_vector(31 downto 0);
signal mt_cpu_csr_hmqo_inten_lo_int             : std_logic_vector(31 downto 0);
signal mt_cpu_csr_hmqo_inten_hi_int             : std_logic_vector(31 downto 0);
signal mt_cpu_csr_core_sel_int                  : std_logic_vector(3 downto 0);
signal mt_cpu_csr_uaddr_addr_int                : std_logic_vector(19 downto 0);
signal mt_cpu_csr_uart_imsk_enable_int          : std_logic_vector(7 downto 0);
signal mt_cpu_csr_dbg_force_int                 : std_logic_vector(7 downto 0);
signal ack_sreg                                 : std_logic_vector(9 downto 0);
signal rddata_reg                               : std_logic_vector(31 downto 0);
signal wrdata_reg                               : std_logic_vector(31 downto 0);
signal bwsel_reg                                : std_logic_vector(3 downto 0);
signal rwaddr_reg                               : std_logic_vector(6 downto 0);
signal ack_in_progress                          : std_logic      ;
signal wr_int                                   : std_logic      ;
signal rd_int                                   : std_logic      ;
signal allones                                  : std_logic_vector(31 downto 0);
signal allzeros                                 : std_logic_vector(31 downto 0);

begin
-- Some internal signals assignments
  wrdata_reg <= wb_dat_i;
-- 
-- Main register bank access process.
  process (clk_sys_i, rst_n_i)
  begin
    if (rst_n_i = '0') then 
      ack_sreg <= "0000000000";
      ack_in_progress <= '0';
      rddata_reg <= "00000000000000000000000000000000";
      mt_cpu_csr_reset_int <= "11111111";
      regs_o.int_load_o <= '0';
      ntf_irq_read <= '0';
      mt_cpu_csr_smem_op_int <= "000";
      mt_cpu_csr_hmq_sel_queue_int <= "000";
      mt_cpu_csr_hmq_sel_core_int <= "000";
      mt_cpu_csr_hmqi_inten_lo_int <= "00000000000000000000000000000000";
      mt_cpu_csr_hmqi_inten_hi_int <= "00000000000000000000000000000000";
      mt_cpu_csr_hmqo_inten_lo_int <= "00000000000000000000000000000000";
      mt_cpu_csr_hmqo_inten_hi_int <= "00000000000000000000000000000000";
      mt_cpu_csr_core_sel_int <= "0000";
      mt_cpu_csr_uaddr_addr_int <= "00000000000000000000";
      regs_o.udata_load_o <= '0';
      uart_msg_data_rd_ack_o <= '0';
      mt_cpu_csr_uart_imsk_enable_int <= "00000000";
      mt_cpu_csr_dbg_force_int <= "00000000";
      regs_o.dbg_core0_insn_wr_o <= '0';
      regs_o.dbg_core1_insn_wr_o <= '0';
      regs_o.dbg_core0_mbx_load_o <= '0';
      regs_o.dbg_core1_mbx_load_o <= '0';
    elsif rising_edge(clk_sys_i) then
-- advance the ACK generator shift register
      ack_sreg(8 downto 0) <= ack_sreg(9 downto 1);
      ack_sreg(9) <= '0';
      if (ack_in_progress = '1') then
        if (ack_sreg(0) = '1') then
          regs_o.int_load_o <= '0';
          ntf_irq_read <= '0';
          regs_o.udata_load_o <= '0';
          uart_msg_data_rd_ack_o <= '0';
          regs_o.dbg_core0_insn_wr_o <= '0';
          regs_o.dbg_core1_insn_wr_o <= '0';
          regs_o.dbg_core0_mbx_load_o <= '0';
          regs_o.dbg_core1_mbx_load_o <= '0';
          ack_in_progress <= '0';
        else
          regs_o.int_load_o <= '0';
          regs_o.udata_load_o <= '0';
          regs_o.dbg_core0_insn_wr_o <= '0';
          regs_o.dbg_core1_insn_wr_o <= '0';
          regs_o.dbg_core0_mbx_load_o <= '0';
          regs_o.dbg_core1_mbx_load_o <= '0';
        end if;
      else
        if ((wb_cyc_i = '1') and (wb_stb_i = '1')) then
          case rwaddr_reg(6 downto 0) is
          when "0000000" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_reset_int <= wrdata_reg(7 downto 0);
            end if;
            rddata_reg(7 downto 0) <= mt_cpu_csr_reset_int;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0000001" => 
            if (wb_we_i = '1') then
              regs_o.int_load_o <= '1';
            end if;
            rddata_reg(7 downto 0) <= regs_i.int_i;
            ntf_irq_read <= '1';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0000010" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.int_val_lo_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0000011" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.int_val_hi_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0000110" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_smem_op_int <= wrdata_reg(2 downto 0);
            end if;
            rddata_reg(2 downto 0) <= mt_cpu_csr_smem_op_int;
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0000111" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_hmq_sel_queue_int <= wrdata_reg(2 downto 0);
              mt_cpu_csr_hmq_sel_core_int <= wrdata_reg(10 downto 8);
            end if;
            rddata_reg(2 downto 0) <= mt_cpu_csr_hmq_sel_queue_int;
            rddata_reg(10 downto 8) <= mt_cpu_csr_hmq_sel_core_int;
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0010000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.hmqi_status_lo_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0010010" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.hmqi_status_hi_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0011000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.hmqo_status_lo_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0011010" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.hmqo_status_hi_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0100000" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_hmqi_inten_lo_int <= wrdata_reg(31 downto 0);
            end if;
            rddata_reg(31 downto 0) <= mt_cpu_csr_hmqi_inten_lo_int;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0100010" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_hmqi_inten_hi_int <= wrdata_reg(31 downto 0);
            end if;
            rddata_reg(31 downto 0) <= mt_cpu_csr_hmqi_inten_hi_int;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0100100" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_hmqo_inten_lo_int <= wrdata_reg(31 downto 0);
            end if;
            rddata_reg(31 downto 0) <= mt_cpu_csr_hmqo_inten_lo_int;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0100110" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_hmqo_inten_hi_int <= wrdata_reg(31 downto 0);
            end if;
            rddata_reg(31 downto 0) <= mt_cpu_csr_hmqo_inten_hi_int;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0110000" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_core_sel_int <= wrdata_reg(3 downto 0);
            end if;
            rddata_reg(3 downto 0) <= mt_cpu_csr_core_sel_int;
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0110001" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_uaddr_addr_int <= wrdata_reg(19 downto 0);
            end if;
            rddata_reg(19 downto 0) <= mt_cpu_csr_uaddr_addr_int;
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "0110010" => 
            if (wb_we_i = '1') then
              regs_o.udata_load_o <= '1';
            end if;
            rddata_reg(31 downto 0) <= regs_i.udata_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1000000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(7 downto 0) <= regs_i.uart_msg_data_i;
            uart_msg_data_rd_ack_o <= '1';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1000001" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(7 downto 0) <= regs_i.uart_poll_ready_i;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1000010" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_uart_imsk_enable_int <= wrdata_reg(7 downto 0);
            end if;
            rddata_reg(7 downto 0) <= mt_cpu_csr_uart_imsk_enable_int;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1100000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(7 downto 0) <= regs_i.dbg_status_i;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1100001" => 
            if (wb_we_i = '1') then
              mt_cpu_csr_dbg_force_int <= wrdata_reg(7 downto 0);
            end if;
            rddata_reg(7 downto 0) <= mt_cpu_csr_dbg_force_int;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1100010" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(7 downto 0) <= regs_i.dbg_insn_ready_i;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1100011" => 
            if (wb_we_i = '1') then
              regs_o.dbg_core0_insn_wr_o <= '1';
            end if;
            rddata_reg(0) <= '0';
            rddata_reg(1) <= '0';
            rddata_reg(2) <= '0';
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1100100" => 
            if (wb_we_i = '1') then
              regs_o.dbg_core1_insn_wr_o <= '1';
            end if;
            rddata_reg(0) <= '0';
            rddata_reg(1) <= '0';
            rddata_reg(2) <= '0';
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1100101" => 
            if (wb_we_i = '1') then
              regs_o.dbg_core0_mbx_load_o <= '1';
            end if;
            rddata_reg(31 downto 0) <= regs_i.dbg_core0_mbx_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "1100110" => 
            if (wb_we_i = '1') then
              regs_o.dbg_core1_mbx_load_o <= '1';
            end if;
            rddata_reg(31 downto 0) <= regs_i.dbg_core1_mbx_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when others =>
-- prevent the slave from hanging the bus on invalid address
            ack_in_progress <= '1';
            ack_sreg(0) <= '1';
          end case;
        end if;
      end if;
    end if;
  end process;
  
  
-- Drive the data output bus
  wb_dat_o <= rddata_reg;
-- Core reset lines
  regs_o.reset_o <= mt_cpu_csr_reset_int;
-- Notification interrupt lines
  regs_o.int_o <= wrdata_reg(7 downto 0);
-- 8-bit notification values from cores 0-3.
-- 8-bit otification values from cores 4-7.
-- Operation code
  regs_o.smem_op_o <= mt_cpu_csr_smem_op_int;
-- Queue select
  regs_o.hmq_sel_queue_o <= mt_cpu_csr_hmq_sel_queue_int;
-- Core select
  regs_o.hmq_sel_core_o <= mt_cpu_csr_hmq_sel_core_int;
-- HMQ IN status core 0-3.
-- HMQ IN status core 4-7.
-- HMQ OUT status core 0-3.
-- HMQ OUT status core 4-7.
-- HMQ IN interrupt enable for core 0-3.
  regs_o.hmqi_inten_lo_o <= mt_cpu_csr_hmqi_inten_lo_int;
-- HMQ IN interrupt enable for core 4-7.
  regs_o.hmqi_inten_hi_o <= mt_cpu_csr_hmqi_inten_hi_int;
-- HMQ OUT interrupt enable for core 0-3.
  regs_o.hmqo_inten_lo_o <= mt_cpu_csr_hmqo_inten_lo_int;
-- HMQ OUT interrupt enable for core core 4-7.
  regs_o.hmqo_inten_hi_o <= mt_cpu_csr_hmqo_inten_hi_int;
-- Select core to access.
  regs_o.core_sel_o <= mt_cpu_csr_core_sel_int;
-- Address to access in selected core's local memory.
  regs_o.uaddr_addr_o <= mt_cpu_csr_uaddr_addr_int;
-- Read/Write data from/to selected core's local memory.
  regs_o.udata_o <= wrdata_reg(31 downto 0);
-- Serial console message byte for the selected core
-- Serial console message data available
-- Per-Core Serial console message Interrupt Enable
  regs_o.uart_imsk_enable_o <= mt_cpu_csr_uart_imsk_enable_int;
-- Per Core debug mode bit
-- Core debug force
  regs_o.dbg_force_o <= mt_cpu_csr_dbg_force_int;
-- Core instruction ready
-- Instruction to be executed
-- pass-through field: Instruction to be executed in register: Debug Interface Core[0] Instruction Register
  regs_o.dbg_core0_insn_o <= wrdata_reg(31 downto 0);
-- Instruction to be executed
-- pass-through field: Instruction to be executed in register: Debug Interface Core[1] Instruction Register
  regs_o.dbg_core1_insn_o <= wrdata_reg(31 downto 0);
-- Mailbox data
  regs_o.dbg_core0_mbx_o <= wrdata_reg(31 downto 0);
-- Mailbox data
  regs_o.dbg_core1_mbx_o <= wrdata_reg(31 downto 0);
  rwaddr_reg <= wb_adr_i;
  wb_stall_o <= (not ack_sreg(0)) and (wb_stb_i and wb_cyc_i);
  wb_err_o <= '0';
  wb_rty_o <= '0';
-- ACK signal generation. Just pass the LSB of ACK counter.
  wb_ack_o <= ack_sreg(0);
end syn;
