-- SPDX-FileCopyrightText: 2022 CERN (home.cern)
--
-- SPDX-License-Identifier: CERN-OHL-W-2.0+

---------------------------------------------------------------------------------------
-- Title          : Wishbone slave core for Mock Turtle CPU Per-Core Local Registers
---------------------------------------------------------------------------------------
-- File           : mt_cpu_lr_wb.vhd
-- Author         : auto-generated by wbgen2 from mt_cpu_lr.wb
-- Created        : Mon Jun 11 18:11:01 2018
-- Standard       : VHDL'87
---------------------------------------------------------------------------------------
-- THIS FILE WAS GENERATED BY wbgen2 FROM SOURCE FILE mt_cpu_lr.wb
-- DO NOT HAND-EDIT UNLESS IT'S ABSOLUTELY NECESSARY!
---------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.mt_cpu_lr_wbgen2_pkg.all;


entity mt_cpu_lr_wb_slave is
  port (
    rst_n_i                                  : in     std_logic;
    clk_sys_i                                : in     std_logic;
    wb_adr_i                                 : in     std_logic_vector(5 downto 0);
    wb_dat_i                                 : in     std_logic_vector(31 downto 0);
    wb_dat_o                                 : out    std_logic_vector(31 downto 0);
    wb_cyc_i                                 : in     std_logic;
    wb_sel_i                                 : in     std_logic_vector(3 downto 0);
    wb_stb_i                                 : in     std_logic;
    wb_we_i                                  : in     std_logic;
    wb_ack_o                                 : out    std_logic;
    wb_err_o                                 : out    std_logic;
    wb_rty_o                                 : out    std_logic;
    wb_stall_o                               : out    std_logic;
    tai_sec_rd_ack_o                         : out    std_logic;
    regs_i                                   : in     t_mt_cpu_lr_in_registers;
    regs_o                                   : out    t_mt_cpu_lr_out_registers
  );
end mt_cpu_lr_wb_slave;

architecture syn of mt_cpu_lr_wb_slave is

signal ack_sreg                                 : std_logic_vector(9 downto 0);
signal rddata_reg                               : std_logic_vector(31 downto 0);
signal wrdata_reg                               : std_logic_vector(31 downto 0);
signal bwsel_reg                                : std_logic_vector(3 downto 0);
signal rwaddr_reg                               : std_logic_vector(5 downto 0);
signal ack_in_progress                          : std_logic      ;
signal wr_int                                   : std_logic      ;
signal rd_int                                   : std_logic      ;
signal allones                                  : std_logic_vector(31 downto 0);
signal allzeros                                 : std_logic_vector(31 downto 0);

begin
-- Some internal signals assignments
  wrdata_reg <= wb_dat_i;
-- 
-- Main register bank access process.
  process (clk_sys_i, rst_n_i)
  begin
    if (rst_n_i = '0') then 
      ack_sreg <= "0000000000";
      ack_in_progress <= '0';
      rddata_reg <= "00000000000000000000000000000000";
      regs_o.ntf_int_load_o <= '0';
      regs_o.uart_chr_wr_o <= '0';
      tai_sec_rd_ack_o <= '0';
      regs_o.delay_cnt_load_o <= '0';
      regs_o.gpio_set_wr_o <= '0';
      regs_o.gpio_clear_wr_o <= '0';
    elsif rising_edge(clk_sys_i) then
-- advance the ACK generator shift register
      ack_sreg(8 downto 0) <= ack_sreg(9 downto 1);
      ack_sreg(9) <= '0';
      if (ack_in_progress = '1') then
        if (ack_sreg(0) = '1') then
          regs_o.ntf_int_load_o <= '0';
          regs_o.uart_chr_wr_o <= '0';
          tai_sec_rd_ack_o <= '0';
          regs_o.delay_cnt_load_o <= '0';
          regs_o.gpio_set_wr_o <= '0';
          regs_o.gpio_clear_wr_o <= '0';
          ack_in_progress <= '0';
        else
          regs_o.ntf_int_load_o <= '0';
          regs_o.uart_chr_wr_o <= '0';
          regs_o.delay_cnt_load_o <= '0';
          regs_o.gpio_set_wr_o <= '0';
          regs_o.gpio_clear_wr_o <= '0';
        end if;
      else
        if ((wb_cyc_i = '1') and (wb_stb_i = '1')) then
          case rwaddr_reg(5 downto 0) is
          when "000000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(3 downto 0) <= regs_i.stat_core_id_i;
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "000001" => 
            if (wb_we_i = '1') then
              regs_o.ntf_int_load_o <= '1';
            end if;
            rddata_reg(7 downto 0) <= regs_i.ntf_int_i;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "000010" => 
            if (wb_we_i = '1') then
              regs_o.uart_chr_wr_o <= '1';
            end if;
            rddata_reg(0) <= '0';
            rddata_reg(1) <= '0';
            rddata_reg(2) <= '0';
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "010000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(7 downto 0) <= regs_i.hmq_stat_in_i;
            rddata_reg(23 downto 16) <= regs_i.hmq_stat_out_i;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "010001" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(7 downto 0) <= regs_i.rmq_stat_in_i;
            rddata_reg(23 downto 16) <= regs_i.rmq_stat_out_i;
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "100000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(0) <= regs_i.wr_stat_link_ok_i;
            rddata_reg(1) <= regs_i.wr_stat_time_ok_i;
            rddata_reg(15 downto 8) <= regs_i.wr_stat_aux_clock_ok_i;
            rddata_reg(2) <= '0';
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "100001" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(27 downto 0) <= regs_i.tai_cycles_i;
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "100010" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.tai_sec_i;
            tai_sec_rd_ack_o <= '1';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "100011" => 
            if (wb_we_i = '1') then
              regs_o.delay_cnt_load_o <= '1';
            end if;
            rddata_reg(31 downto 0) <= regs_i.delay_cnt_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "110000" => 
            if (wb_we_i = '1') then
            end if;
            rddata_reg(31 downto 0) <= regs_i.gpio_in_i;
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "110001" => 
            if (wb_we_i = '1') then
              regs_o.gpio_set_wr_o <= '1';
            end if;
            rddata_reg(0) <= '0';
            rddata_reg(1) <= '0';
            rddata_reg(2) <= '0';
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when "110010" => 
            if (wb_we_i = '1') then
              regs_o.gpio_clear_wr_o <= '1';
            end if;
            rddata_reg(0) <= '0';
            rddata_reg(1) <= '0';
            rddata_reg(2) <= '0';
            rddata_reg(3) <= '0';
            rddata_reg(4) <= '0';
            rddata_reg(5) <= '0';
            rddata_reg(6) <= '0';
            rddata_reg(7) <= '0';
            rddata_reg(8) <= '0';
            rddata_reg(9) <= '0';
            rddata_reg(10) <= '0';
            rddata_reg(11) <= '0';
            rddata_reg(12) <= '0';
            rddata_reg(13) <= '0';
            rddata_reg(14) <= '0';
            rddata_reg(15) <= '0';
            rddata_reg(16) <= '0';
            rddata_reg(17) <= '0';
            rddata_reg(18) <= '0';
            rddata_reg(19) <= '0';
            rddata_reg(20) <= '0';
            rddata_reg(21) <= '0';
            rddata_reg(22) <= '0';
            rddata_reg(23) <= '0';
            rddata_reg(24) <= '0';
            rddata_reg(25) <= '0';
            rddata_reg(26) <= '0';
            rddata_reg(27) <= '0';
            rddata_reg(28) <= '0';
            rddata_reg(29) <= '0';
            rddata_reg(30) <= '0';
            rddata_reg(31) <= '0';
            ack_sreg(0) <= '1';
            ack_in_progress <= '1';
          when others =>
-- prevent the slave from hanging the bus on invalid address
            ack_in_progress <= '1';
            ack_sreg(0) <= '1';
          end case;
        end if;
      end if;
    end if;
  end process;
  
  
-- Drive the data output bus
  wb_dat_o <= rddata_reg;
-- ID (number) of the CPU core owning this register.
-- Write a value to send a notification interrupt to the host.
  regs_o.ntf_int_o <= wrdata_reg(7 downto 0);
-- Write port for serial console.
-- pass-through field: Write port for serial console. in register: Serial Console Output
  regs_o.uart_chr_o <= wrdata_reg(7 downto 0);
-- HMQ IN Slot Status
-- HMQ OUT Slot Status
-- RMQ IN Slot Status
-- RMQ OUT Slot Status
-- WR Link Up
-- WR Time OK
-- WR Aux Clock OK
-- When White Rabbit is enabled, this returns the TAI clock ticks.
-- When White Rabbit is enabled, this returns the TAI seconds.
-- Counts down at every system clock cycle and stops at 0.
  regs_o.delay_cnt_o <= wrdata_reg(31 downto 0);
-- GPIO In
-- GPIO Set
-- pass-through field: GPIO Set in register: GPIO Set
  regs_o.gpio_set_o <= wrdata_reg(31 downto 0);
-- GPIO Clear
-- pass-through field: GPIO Clear in register: GPIO Clear
  regs_o.gpio_clear_o <= wrdata_reg(31 downto 0);
  rwaddr_reg <= wb_adr_i;
  wb_stall_o <= (not ack_sreg(0)) and (wb_stb_i and wb_cyc_i);
  wb_err_o <= '0';
  wb_rty_o <= '0';
-- ACK signal generation. Just pass the LSB of ACK counter.
  wb_ack_o <= ack_sreg(0);
end syn;
