/*
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 * SPDX-FileCopyrightText: 2019 CERN
 */

SECTIONS
{

  /*--------------------------------------------------------------------*/
  /* Code and read-only segment                                         */
  /*--------------------------------------------------------------------*/

  /* Begining of code and text segment */
  . = 0x00000000;
  _text_start = .;
  PROVIDE( eprol = . );

  /* text: Program code section */
  .text :
  {
    *(.boot)
    *(.text)
    *(.text.*)
    *(.gnu.linkonce.t.*)
  } > ram

  /* init: Code to execute before main (called by crt0.S) */
  .init :
  {
    KEEP( *(.init) )
  } > ram

  /* fini: Code to execute after main (called by crt0.S) */
  .fini :
  {
    KEEP( *(.fini) )
  } > ram

  /* rodata: Read-only data */
  .rodata :
  {
    *(.rdata)
    *(.rodata)
    *(.rodata.*)
    *(.gnu.linkonce.r.*)
  } > ram

  /* End of code and read-only segment */
  PROVIDE( etext = . );
  _etext = .;

  /*--------------------------------------------------------------------*/
  /* Global constructor/destructor segement                             */
  /*--------------------------------------------------------------------*/
  /* The .ctors/.dtors sections are special sections which contain a
     list of constructor/destructor function pointers. crtbegin.o
     includes code in a .init section which goes through the .ctors list
     and calls each constuctor. crtend.o includes code in a .fini
     section which goes through the .dtors list and calls each
     destructor. crtbegin.o includes a special null pointer in its own
     .ctors/.dtors sections which acts as a start indicator for those
     lists. crtend.o also includes a special null pointer in its own
     .ctors/.dtors sections which acts as an end indictor. The linker
     commands below are setup so that crtbegin.o's .ctors/.dtors
     sections are always first and crtend.o's .ctors/.dtors sections are
     always last. This is the only way the list of functions will have
     the begin and end indicators in the right place. */

  /* ctors : Array of global constructor function pointers */

  /*--------------------------------------------------------------------*/
  /* Initialized data segment                                           */
  /*--------------------------------------------------------------------*/


  /* data: Writable data */
  .data : ALIGN(4)
  {
    /* Start of initialized data segment */
    _data_start = .;
    *(.data)
    *(.data.*)
    *(.gnu.linkonce.d.*)
  } > ram

  /* Writable small data segment */
  .sdata : ALIGN(4)
  {
    /* 12-bit signed offsets from global pointer */
    __global_pointer$ = . + 0x800;
    *(.sdata)
    *(.sdata.*)
    *(.srodata.*)
    *(.gnu.linkonce.s.*)
  } > ram

    /* End of initialized data segment */
    PROVIDE( edata = . );
    _edata = .;

  /*--------------------------------------------------------------------*/
  /* Uninitialized data segment                                         */
  /*--------------------------------------------------------------------*/


  /* Writable uninitialized small data segment */
  .sbss : ALIGN(4)
  {
    /* Start of uninitialized data segment */
    _bss_start = .;
    *(.sbss)
    *(.sbss.*)
    *(.gnu.linkonce.sb.*)
  } > ram

  /* bss: Uninitialized writeable data section */
  .bss :
  {
    *(.bss)
    *(.bss.*)
    *(.gnu.linkonce.b.*)
    *(COMMON)
  } > ram


  /* End of uninitialized data segment (used by syscalls.c for heap) */
  PROVIDE( end = . );
  _end = ALIGN(8);

  .smem : { *(.smem) } > smem

 PROVIDE(_endram = ORIGIN(stack));
 PROVIDE(_fexception_stack = ORIGIN(stack) + LENGTH(stack) - 4);
 PROVIDE(_fstack = ORIGIN(stack) + LENGTH(stack) - 4 - 0x400 );
}
